# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: animalai/communicator_objects/arenas_configurations_proto.proto

import sys

_b = sys.version_info[0] < 3 and (lambda x: x) or (lambda x: x.encode("latin1"))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from animalai.communicator_objects import (
    arena_configuration_proto_pb2 as animalai_dot_communicator__objects_dot_arena__configuration__proto__pb2,
)


DESCRIPTOR = _descriptor.FileDescriptor(
    name="animalai/communicator_objects/arenas_configurations_proto.proto",
    package="communicator_objects",
    syntax="proto3",
    serialized_pb=_b(
        '\n?animalai/communicator_objects/arenas_configurations_proto.proto\x12\x14\x63ommunicator_objects\x1a=animalai/communicator_objects/arena_configuration_proto.proto"\xd4\x01\n\x19\x41renasConfigurationsProto\x12K\n\x06\x61renas\x18\x01 \x03(\x0b\x32;.communicator_objects.ArenasConfigurationsProto.ArenasEntry\x12\x0c\n\x04seed\x18\x02 \x01(\x05\x1a\\\n\x0b\x41renasEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12<\n\x05value\x18\x02 \x01(\x0b\x32-.communicator_objects.ArenaConfigurationProto:\x02\x38\x01\x42\x14\xaa\x02\x11\x41\x41IOCommunicatorsb\x06proto3'
    ),
    dependencies=[
        animalai_dot_communicator__objects_dot_arena__configuration__proto__pb2.DESCRIPTOR,
    ],
)


_ARENASCONFIGURATIONSPROTO_ARENASENTRY = _descriptor.Descriptor(
    name="ArenasEntry",
    full_name="communicator_objects.ArenasConfigurationsProto.ArenasEntry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="communicator_objects.ArenasConfigurationsProto.ArenasEntry.key",
            index=0,
            number=1,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="communicator_objects.ArenasConfigurationsProto.ArenasEntry.value",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b("8\001")),
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=273,
    serialized_end=365,
)

_ARENASCONFIGURATIONSPROTO = _descriptor.Descriptor(
    name="ArenasConfigurationsProto",
    full_name="communicator_objects.ArenasConfigurationsProto",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="arenas",
            full_name="communicator_objects.ArenasConfigurationsProto.arenas",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="seed",
            full_name="communicator_objects.ArenasConfigurationsProto.seed",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[_ARENASCONFIGURATIONSPROTO_ARENASENTRY,],
    enum_types=[],
    options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=153,
    serialized_end=365,
)

_ARENASCONFIGURATIONSPROTO_ARENASENTRY.fields_by_name[
    "value"
].message_type = (
    animalai_dot_communicator__objects_dot_arena__configuration__proto__pb2._ARENACONFIGURATIONPROTO
)
_ARENASCONFIGURATIONSPROTO_ARENASENTRY.containing_type = _ARENASCONFIGURATIONSPROTO
_ARENASCONFIGURATIONSPROTO.fields_by_name[
    "arenas"
].message_type = _ARENASCONFIGURATIONSPROTO_ARENASENTRY
DESCRIPTOR.message_types_by_name[
    "ArenasConfigurationsProto"
] = _ARENASCONFIGURATIONSPROTO
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ArenasConfigurationsProto = _reflection.GeneratedProtocolMessageType(
    "ArenasConfigurationsProto",
    (_message.Message,),
    dict(
        ArenasEntry=_reflection.GeneratedProtocolMessageType(
            "ArenasEntry",
            (_message.Message,),
            dict(
                DESCRIPTOR=_ARENASCONFIGURATIONSPROTO_ARENASENTRY,
                __module__="animalai.communicator_objects.arenas_configurations_proto_pb2"
                # @@protoc_insertion_point(class_scope:communicator_objects.ArenasConfigurationsProto.ArenasEntry)
            ),
        ),
        DESCRIPTOR=_ARENASCONFIGURATIONSPROTO,
        __module__="animalai.communicator_objects.arenas_configurations_proto_pb2"
        # @@protoc_insertion_point(class_scope:communicator_objects.ArenasConfigurationsProto)
    ),
)
_sym_db.RegisterMessage(ArenasConfigurationsProto)
_sym_db.RegisterMessage(ArenasConfigurationsProto.ArenasEntry)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(
    descriptor_pb2.FileOptions(), _b("\252\002\021AAIOCommunicators")
)
_ARENASCONFIGURATIONSPROTO_ARENASENTRY.has_options = True
_ARENASCONFIGURATIONSPROTO_ARENASENTRY._options = _descriptor._ParseOptions(
    descriptor_pb2.MessageOptions(), _b("8\001")
)
# @@protoc_insertion_point(module_scope)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: animalai/communicator_objects/vector_proto.proto

import sys

_b = sys.version_info[0] < 3 and (lambda x: x) or (lambda x: x.encode("latin1"))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="animalai/communicator_objects/vector_proto.proto",
    package="communicator_objects",
    syntax="proto3",
    serialized_pb=_b(
        '\n0animalai/communicator_objects/vector_proto.proto\x12\x14\x63ommunicator_objects".\n\x0bVectorProto\x12\t\n\x01x\x18\x01 \x01(\x02\x12\t\n\x01y\x18\x02 \x01(\x02\x12\t\n\x01z\x18\x03 \x01(\x02\x42\x14\xaa\x02\x11\x41\x41IOCommunicatorsb\x06proto3'
    ),
)


_VECTORPROTO = _descriptor.Descriptor(
    name="VectorProto",
    full_name="communicator_objects.VectorProto",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="x",
            full_name="communicator_objects.VectorProto.x",
            index=0,
            number=1,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="y",
            full_name="communicator_objects.VectorProto.y",
            index=1,
            number=2,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="z",
            full_name="communicator_objects.VectorProto.z",
            index=2,
            number=3,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=74,
    serialized_end=120,
)

DESCRIPTOR.message_types_by_name["VectorProto"] = _VECTORPROTO
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

VectorProto = _reflection.GeneratedProtocolMessageType(
    "VectorProto",
    (_message.Message,),
    dict(
        DESCRIPTOR=_VECTORPROTO,
        __module__="animalai.communicator_objects.vector_proto_pb2"
        # @@protoc_insertion_point(class_scope:communicator_objects.VectorProto)
    ),
)
_sym_db.RegisterMessage(VectorProto)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(
    descriptor_pb2.FileOptions(), _b("\252\002\021AAIOCommunicators")
)
# @@protoc_insertion_point(module_scope)



import uuid
from typing import NamedTuple
from typing import Optional, List
from mlagents_envs.environment import UnityEnvironment
from mlagents_envs.rpc_communicator import UnityTimeOutException
from mlagents_envs.side_channel.raw_bytes_channel import RawBytesChannel
from mlagents_envs.side_channel.side_channel import SideChannel
from mlagents_envs.side_channel.engine_configuration_channel import (
    EngineConfig,
    EngineConfigurationChannel,
)
from animalai.envs.arena_config import ArenaConfig


class PlayTrain(NamedTuple):
    play: int
    train: int


class AnimalAIEnvironment(UnityEnvironment):
    # Default values for configuration parameters of the environment, can be changed if needed
    # Increasing the timescale value for training might speed up the process on powefull machines
    # but take care as the higher the timescale the more likely the physics might break
    WINDOW_WIDTH = PlayTrain(play=1200, train=80)
    WINDOW_HEIGHT = PlayTrain(play=800, train=80)
    QUALITY_LEVEL = PlayTrain(play=5, train=1)
    TIMESCALE = PlayTrain(play=1, train=300)
    TARGET_FRAME_RATE = PlayTrain(play=60, train=-1)
    ARENA_CONFIG_SC_UUID = "9c36c837-cad5-498a-b675-bc19c9370072"

    def __init__(
        self,
        file_name: Optional[str] = None,
        worker_id: int = 0,
        base_port: int = 5005,
        seed: int = 0,
        # docker_training: bool = False, # Will be removed in final version
        n_arenas: int = 1,
        play: bool = False,
        arenas_configurations: ArenaConfig = None,
        inference: bool = False,
        resolution: int = None,
        grayscale: bool = False,
        side_channels: Optional[List[SideChannel]] = None,
    ):

        args = self.executable_args(n_arenas, play, resolution, grayscale)
        self.play = play
        self.inference = inference
        self.timeout = 10 if play else 60
        self.side_channels = side_channels if side_channels else []
        self.arenas_parameters_side_channel = None

        self.configure_side_channels(self.side_channels)

        super().__init__(
            file_name=file_name,
            worker_id=worker_id,
            base_port=base_port,
            seed=seed,
            no_graphics=False,
            timeout_wait=self.timeout,
            args=args,
            side_channels=self.side_channels,
        )
        self.reset(arenas_configurations)

    def configure_side_channels(self, side_channels: List[SideChannel]) -> None:

        contains_engine_config_sc = any(
            [isinstance(sc, EngineConfigurationChannel) for sc in side_channels]
        )
        if not contains_engine_config_sc:
            self.side_channels.append(self.create_engine_config_side_channel())
        contains_arena_config_sc = any(
            [sc.channel_id == self.ARENA_CONFIG_SC_UUID for sc in side_channels]
        )
        if not contains_arena_config_sc:
            self.arenas_parameters_side_channel = RawBytesChannel(
                channel_id=uuid.UUID(self.ARENA_CONFIG_SC_UUID)
            )
            self.side_channels.append(self.arenas_parameters_side_channel)

    def create_engine_config_side_channel(self) -> EngineConfigurationChannel:

        if self.play or self.inference:
            engine_configuration = EngineConfig(
                width=self.WINDOW_WIDTH.play,
                height=self.WINDOW_HEIGHT.play,
                quality_level=self.QUALITY_LEVEL.play,
                time_scale=self.TIMESCALE.play,
                target_frame_rate=self.TARGET_FRAME_RATE.play,
            )
        else:
            engine_configuration = EngineConfig(
                width=self.WINDOW_WIDTH.train,
                height=self.WINDOW_HEIGHT.train,
                quality_level=self.QUALITY_LEVEL.train,
                time_scale=self.TIMESCALE.train,
                target_frame_rate=self.TARGET_FRAME_RATE.train,
            )
        engine_configuration_channel = EngineConfigurationChannel()
        engine_configuration_channel.set_configuration(engine_configuration)
        return engine_configuration_channel

    def reset(self, arenas_configurations: ArenaConfig = None) -> None:
        if arenas_configurations:
            arenas_configurations_proto = arenas_configurations.to_proto()
            arenas_configurations_proto_string = arenas_configurations_proto.SerializeToString(
                deterministic=True
            )
            self.arenas_parameters_side_channel.send_raw_data(
                bytearray(arenas_configurations_proto_string)
            )
        try:
            super().reset()
        except UnityTimeOutException as timeoutException:
            if self.play:
                pass
            else:
                raise timeoutException

    def close(self):
        if self.play:
            self.communicator.close()
            if self.proc1:
                self.proc1.kill()
        else:
            super().close()

    @staticmethod
    def executable_args(
        n_arenas: int = 1,
        play: bool = False,
        resolution: int = 84,
        grayscale: bool = False,
    ) -> List[str]:
        args = ["--playerMode"]
        if play:
            args.append("1")
        else:
            args.append("0")
        args.append("--numberOfArenas")
        args.append(str(n_arenas))
        if resolution:
            args.append("--resolution")
            args.append(str(resolution))
        if grayscale:
            args.append("--grayscale")
        return args



import numpy as np
from gym import spaces
from typing import Union, List

from gym_unity.envs import (
    UnityEnv,
    UnityGymException,
    GymStepResult,
    AgentIdIndexMapper,
    BatchedStepResult,
    ActionFlattener,
    logger,
)

from animalai.envs.environment import AnimalAIEnvironment
from animalai.envs.arena_config import ArenaConfig


class AnimalAIGym(UnityEnv):
    def __init__(
        self,
        environment_filename: str,
        worker_id: int = 0,
        uint8_visual: bool = False,
        flatten_branched: bool = False,
        n_arenas: int = 1,
        seed: int = 0,
        inference: bool = False,
        grayscale: bool = False,
        arenas_configurations: ArenaConfig = None,
    ):
        """
        Environment initialization
        :param environment_filename: The UnityEnvironment path or file to be wrapped in the gym.
        :param worker_id: Worker number for environment.
        :param use_visual: Whether to use visual observation or vector observation.
        :param uint8_visual: Return visual observations as uint8 (0-255) matrices instead of float (0.0-1.0).
        :param multiagent: Whether to run in multi-agent mode (lists of obs, reward, done).
        :param flatten_branched: If True, turn branched discrete action spaces into a Discrete space rather than
            MultiDiscrete.
        :param no_graphics: Whether to run the Unity simulator in no-graphics mode
        :param allow_multiple_visual_obs: If True, return a list of visual observations instead of only one.
        """
        base_port = 5005
        if environment_filename is None:
            base_port = AnimalAIEnvironment.DEFAULT_EDITOR_PORT

        self._env = AnimalAIEnvironment(
            file_name=environment_filename,
            worker_id=worker_id,
            base_port=base_port,
            seed=seed,
            n_arenas=n_arenas,
            inference=inference,
            arenas_configurations=arenas_configurations,
            grayscale=grayscale,
        )

        # Take a single step so that the brain information will be sent over
        if not self._env.get_agent_groups():
            self._env.step()

        self.visual_obs = None
        self._n_agents = -1

        self.agent_mapper = AgentIdIndexMapper()

        # Save the step result from the last time all Agents requested decisions.
        self._previous_step_result: BatchedStepResult = None
        self._multiagent = n_arenas > 1
        self._flattener = None
        # Hidden flag used by Atari environments to determine if the game is over
        self.game_over = False
        self._allow_multiple_visual_obs = n_arenas > 1

        # Check brain configuration
        if len(self._env.get_agent_groups()) != 1:
            raise UnityGymException(
                "There can only be one brain in a UnityEnvironment "
                "if it is wrapped in a gym."
            )

        self.brain_name = self._env.get_agent_groups()[0]
        self.name = self.brain_name
        self.group_spec = self._env.get_agent_group_spec(self.brain_name)

        if self._get_n_vis_obs() == 0:
            raise UnityGymException(
                "`use_visual` was set to True, however there are no"
                " visual observations as part of this environment."
            )
        self.use_visual = self._get_n_vis_obs() >= 1
        self.uint8_visual = uint8_visual

        # Check for number of agents in scene.
        self._env.reset()
        step_result = self._env.get_step_result(self.brain_name)
        self._check_agents(step_result.n_agents())
        self._previous_step_result = step_result
        self.agent_mapper.set_initial_agents(list(self._previous_step_result.agent_id))

        # Set observation and action spaces
        if self.group_spec.is_action_discrete():
            branches = self.group_spec.discrete_action_branches
            if self.group_spec.action_shape == 1:
                self._action_space = spaces.Discrete(branches[0])
            else:
                if flatten_branched:
                    self._flattener = ActionFlattener(branches)
                    self._action_space = self._flattener.action_space
                else:
                    self._action_space = spaces.MultiDiscrete(branches)

        else:
            if flatten_branched:
                logger.warning(
                    "The environment has a non-discrete action space. It will "
                    "not be flattened."
                )
            high = np.array([1] * self.group_spec.action_shape)
            self._action_space = spaces.Box(-high, high, dtype=np.float32)
        high = np.array([np.inf] * self._get_vec_obs_size())
        if self.use_visual:
            shape = self._get_vis_obs_shape()
            if uint8_visual:
                self._observation_space = spaces.Box(
                    0, 255, dtype=np.uint8, shape=shape
                )
            else:
                self._observation_space = spaces.Box(
                    0, 1, dtype=np.float32, shape=shape
                )

        else:
            self._observation_space = spaces.Box(-high, high, dtype=np.float32)

    def reset(
        self, arenas_configurations: ArenaConfig = None
    ) -> Union[List[np.ndarray], np.ndarray]:
        """Resets the state of the environment and returns an initial observation.
        In the case of multi-agent environments, this is a list.
        Returns: observation (object/list): the initial observation of the
        space.
        """
        step_result = self._step(True, arenas_configurations=arenas_configurations)
        n_agents = step_result.n_agents()
        self._check_agents(n_agents)
        self.game_over = False

        if not self._multiagent:
            res: GymStepResult = self._single_step(step_result)
        else:
            res = self._multi_step(step_result)
        return res[0]

    def _step(
        self, needs_reset: bool = False, arenas_configurations: ArenaConfig = None
    ) -> BatchedStepResult:
        if needs_reset:
            self._env.reset(arenas_configurations=arenas_configurations)
        else:
            self._env.step()
        info = self._env.get_step_result(self.brain_name)
        # Two possible cases here:
        # 1) all agents requested decisions (some of which might be done)
        # 2) some Agents were marked Done in between steps.
        # In case 2,  we re-request decisions until all agents request a real decision.
        while info.n_agents() - sum(info.done) < self._n_agents:
            if not info.done.all():
                raise UnityGymException(
                    "The environment does not have the expected amount of agents. "
                    + "Some agents did not request decisions at the same time."
                )
            for agent_id, reward in zip(info.agent_id, info.reward):
                self.agent_mapper.mark_agent_done(agent_id, reward)

            self._env.step()
            info = self._env.get_step_result(self.brain_name)
        return self._sanitize_info(info)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: animalai/communicator_objects/items_to_spawn_proto.proto

import sys

_b = sys.version_info[0] < 3 and (lambda x: x) or (lambda x: x.encode("latin1"))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from animalai.communicator_objects import (
    vector_proto_pb2 as animalai_dot_communicator__objects_dot_vector__proto__pb2,
)


DESCRIPTOR = _descriptor.FileDescriptor(
    name="animalai/communicator_objects/items_to_spawn_proto.proto",
    package="communicator_objects",
    syntax="proto3",
    serialized_pb=_b(
        '\n8animalai/communicator_objects/items_to_spawn_proto.proto\x12\x14\x63ommunicator_objects\x1a\x30\x61nimalai/communicator_objects/vector_proto.proto"\xce\x01\n\x10ItemToSpawnProto\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x34\n\tpositions\x18\x03 \x03(\x0b\x32!.communicator_objects.VectorProto\x12\x11\n\trotations\x18\x04 \x03(\x02\x12\x30\n\x05sizes\x18\x05 \x03(\x0b\x32!.communicator_objects.VectorProto\x12\x31\n\x06\x63olors\x18\x06 \x03(\x0b\x32!.communicator_objects.VectorProtoB\x14\xaa\x02\x11\x41\x41IOCommunicatorsb\x06proto3'
    ),
    dependencies=[
        animalai_dot_communicator__objects_dot_vector__proto__pb2.DESCRIPTOR,
    ],
)


_ITEMTOSPAWNPROTO = _descriptor.Descriptor(
    name="ItemToSpawnProto",
    full_name="communicator_objects.ItemToSpawnProto",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="name",
            full_name="communicator_objects.ItemToSpawnProto.name",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="positions",
            full_name="communicator_objects.ItemToSpawnProto.positions",
            index=1,
            number=3,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="rotations",
            full_name="communicator_objects.ItemToSpawnProto.rotations",
            index=2,
            number=4,
            type=2,
            cpp_type=6,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="sizes",
            full_name="communicator_objects.ItemToSpawnProto.sizes",
            index=3,
            number=5,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="colors",
            full_name="communicator_objects.ItemToSpawnProto.colors",
            index=4,
            number=6,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=133,
    serialized_end=339,
)

_ITEMTOSPAWNPROTO.fields_by_name[
    "positions"
].message_type = animalai_dot_communicator__objects_dot_vector__proto__pb2._VECTORPROTO
_ITEMTOSPAWNPROTO.fields_by_name[
    "sizes"
].message_type = animalai_dot_communicator__objects_dot_vector__proto__pb2._VECTORPROTO
_ITEMTOSPAWNPROTO.fields_by_name[
    "colors"
].message_type = animalai_dot_communicator__objects_dot_vector__proto__pb2._VECTORPROTO
DESCRIPTOR.message_types_by_name["ItemToSpawnProto"] = _ITEMTOSPAWNPROTO
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ItemToSpawnProto = _reflection.GeneratedProtocolMessageType(
    "ItemToSpawnProto",
    (_message.Message,),
    dict(
        DESCRIPTOR=_ITEMTOSPAWNPROTO,
        __module__="animalai.communicator_objects.items_to_spawn_proto_pb2"
        # @@protoc_insertion_point(class_scope:communicator_objects.ItemToSpawnProto)
    ),
)
_sym_db.RegisterMessage(ItemToSpawnProto)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(
    descriptor_pb2.FileOptions(), _b("\252\002\021AAIOCommunicators")
)
# @@protoc_insertion_point(module_scope)



import yaml

from typing import List
from animalai.communicator_objects import (
    ArenasConfigurationsProto,
    ArenaConfigurationProto,
    ItemToSpawnProto,
    VectorProto,
)

yaml.Dumper.ignore_aliases = lambda *args: True


class Vector3(yaml.YAMLObject):
    yaml_tag = u"!Vector3"

    def __init__(self, x: float = 0, y: float = 0, z: float = 0):
        self.x = x
        self.y = y
        self.z = z

    def to_proto(self) -> VectorProto:
        vector_proto = VectorProto()
        vector_proto.x = self.x
        vector_proto.y = self.y
        vector_proto.z = self.z

        return vector_proto


class RGB(yaml.YAMLObject):
    yaml_tag = u"!RGB"

    def __init__(self, r: float = 0, g: float = 0, b: float = 0):
        self.r = r
        self.g = g
        self.b = b

    def to_proto(self) -> VectorProto:
        rgb_proto = VectorProto()
        rgb_proto.x = self.r
        rgb_proto.y = self.g
        rgb_proto.z = self.b

        return rgb_proto


class Item(yaml.YAMLObject):
    yaml_tag = u"!Item"

    def __init__(
        self,
        name: str = "",
        positions: List[Vector3] = None,
        rotations: List[float] = None,
        sizes: List[Vector3] = None,
        colors: List[RGB] = None,
    ):
        self.name = name
        self.positions = positions if positions is not None else []
        self.rotations = rotations if rotations is not None else []
        self.sizes = sizes if sizes is not None else []
        self.colors = colors if colors is not None else []

    def to_proto(self) -> ItemToSpawnProto:
        item_to_spawn_proto = ItemToSpawnProto()
        item_to_spawn_proto.name = self.name
        item_to_spawn_proto.positions.extend([v.to_proto() for v in self.positions])
        item_to_spawn_proto.rotations.extend(self.rotations)
        item_to_spawn_proto.sizes.extend([v.to_proto() for v in self.sizes])
        item_to_spawn_proto.colors.extend([v.to_proto() for v in self.colors])

        return item_to_spawn_proto


class Arena(yaml.YAMLObject):
    yaml_tag = u"!Arena"

    def __init__(
        self,
        t: int = 1000,
        items: List[Item] = None,
        pass_mark: float = 0,
        blackouts: List[int] = None,
    ):
        self.t = t
        self.items = items if items is not None else {}
        self.pass_mark = pass_mark
        self.blackouts = blackouts if blackouts is not None else []

    def to_proto(self) -> ArenaConfigurationProto:
        arena_configuration_proto = ArenaConfigurationProto()
        arena_configuration_proto.t = self.t
        arena_configuration_proto.pass_mark = self.pass_mark
        arena_configuration_proto.blackouts.extend(self.blackouts)
        arena_configuration_proto.items.extend([item.to_proto() for item in self.items])

        return arena_configuration_proto


class ArenaConfig(yaml.YAMLObject):
    yaml_tag = u"!ArenaConfig"

    def __init__(self, yaml_path: str = None):

        if yaml_path is not None:
            self.arenas = yaml.load(open(yaml_path, "r"), Loader=yaml.Loader).arenas
        else:
            self.arenas = {}

    # def save_config(self, json_path: str) -> None:
    #     out = jsonpickle.encode(self.arenas)
    #     out = json.loads(out)
    #     json.dump(out, open(json_path, "w"), indent=4)

    def to_proto(self, seed: int = -1) -> ArenasConfigurationsProto:
        arenas_configurations_proto = ArenasConfigurationsProto()
        arenas_configurations_proto.seed = seed

        for k in self.arenas:
            arenas_configurations_proto.arenas[k].CopyFrom(self.arenas[k].to_proto())

        return arenas_configurations_proto

    # def update(self, arenas_configurations:):
    #
    #     if arenas_configurations is not None:
    #         for arena_i in arenas_configurations.arenas:
    #             self.arenas[arena_i] = copy.copy(arenas_configurations.arenas[arena_i])


def constructor_arena(loader, node):
    fields = loader.construct_mapping(node)
    return Arena(**fields)


def constructor_item(loader, node):
    fields = loader.construct_mapping(node)
    return Item(**fields)


yaml.add_constructor(u"!Arena", constructor_arena)
yaml.add_constructor(u"!Item", constructor_item)



from setuptools import setup, find_packages

setup(
    name="animalai",
    version="2.0.0",
    description="Animal AI environment Python API",
    url="https://github.com/beyretb/AnimalAI-Olympics",
    author="Benjamin Beyret",
    author_email="beyretb@gmail.com",
    classifiers=[
        "Intended Audience :: Developers",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    packages=find_packages(exclude=["*.tests", "*.tests.*", "tests.*", "tests"]),
    zip_safe=False,
    install_requires=["mlagents-envs==0.15.0", "gym-unity==0.15.0", "jsonpickle", "pyyaml"],
    python_requires=">=3.5",
)
